import React, {Component} from "react";
import {FormControl, FormControlLabel, FormGroup, FormLabel, Grid, Switch} from "@mui/material";

type GameSwitch = {
    name: GameSwitchNames,
    label: string,
    depends?: GameSwitchNames,
    value: boolean
}

type GameSwitchNames =
    "GET_BONUS_PASSING_START"
    | "DOUBLE_BONUS_ON_GO"
    | "PUT_PAID_PENALTIES_IN_MIDDLE"
    | "RECEIVE_PENALTIES_AT_FREE_PARKING"
    | "PUT_10_IN_MIDDLE_AFTER_EACH_GO"
    | "STREET_MUST_BE_AUCTIONED_IF_NOT_PURCHASED"
    | "TWO_HOTELS_PER_STREET_ALLOWED"
    | "NO_MORTGAGE_ON_STREETS_WITH_BUILDINGS"
    | "MORTGAGE_REDUCES_RENT"
    | "GOODS_GO_TO_OTHER_PLAYERS_UPON_BANKRUPTCY"
    | "STREETS_CAN_BE_SOLD_BACK_TO_THE_BANK"
    | "PRISON_CAN_BE_PURCHASED_RENT_GOES_TO_PLAYER";

type GameInputs = {
    "PASSING_GO_CASH": string,
    "STARTING_CASH": string,
    "PRISON_RELEASE_FEE": string
}

interface InitialSetupState {
    open: boolean,
    switchValues: GameSwitch[],
    inputValues: GameInputs
}

export default class Setup extends Component<{}, InitialSetupState> {
    constructor(props: {}) {
        super(props);
        this.state = {
            open: true,
            switchValues: [
                {
                    name: "GET_BONUS_PASSING_START",
                    label: "Über Los Bonus einziehen",
                    value: true
                },
                {
                    name: "DOUBLE_BONUS_ON_GO",
                    label: "Wenn auf Los, doppelten Bonus einziehen",
                    depends: "GET_BONUS_PASSING_START",
                    value: true
                },
                {
                    name: "PUT_PAID_PENALTIES_IN_MIDDLE",
                    label: "Gezahlte Strafen in die 'Mitte' legen",
                    value: true
                },
                {
                    name: "RECEIVE_PENALTIES_AT_FREE_PARKING",
                    label: "Bei 'Frei Parken' erhaltene Strafen",
                    depends: "PUT_PAID_PENALTIES_IN_MIDDLE",
                    value: true
                },
                {
                    name: "PUT_10_IN_MIDDLE_AFTER_EACH_GO",
                    label: "Nach jedem Los 10 in die Mitte",
                    depends: "PUT_PAID_PENALTIES_IN_MIDDLE",
                    value: false
                },
                {
                    name: "STREET_MUST_BE_AUCTIONED_IF_NOT_PURCHASED",
                    label: "Straße muss versteigert werden bei Nichtkauf",
                    value: true
                },
                {
                    name: "TWO_HOTELS_PER_STREET_ALLOWED",
                    label: "Zwei Hotels pro Straße erlaubt",
                    value: false
                },
                {
                    name: "NO_MORTGAGE_ON_STREETS_WITH_BUILDINGS",
                    label: "Keine Hypothek auf Straßen mit Bauten",
                    value: true
                },
                {
                    name: "MORTGAGE_REDUCES_RENT",
                    label: "Hypothek verringert Miete",
                    value: false
                },
                {
                    name: "GOODS_GO_TO_OTHER_PLAYERS_UPON_BANKRUPTCY",
                    label: "Güter gehen an anderen Spieler über bei Bankrott",
                    value: true
                },
                {
                    name: "STREETS_CAN_BE_SOLD_BACK_TO_THE_BANK",
                    label: "Straßen können zurück an die Bank verkauft werden",
                    value: true
                },
                {
                    name: "PRISON_CAN_BE_PURCHASED_RENT_GOES_TO_PLAYER",
                    label: "Gefängnis kann erworben werden, Miete geht an den Spieler",
                    value: false
                }
            ],
            inputValues: {
                "PASSING_GO_CASH": "200",
                "STARTING_CASH": "1500",
                "PRISON_RELEASE_FEE": "50"
            }
        }
    }

    style = {
        position: 'absolute',
        top: '50%',
        left: '50%',
        transform: 'translate(-50%, -50%)',
        width: '50%',
        bgcolor: 'background.paper',
        border: '2px solid #000',
        boxShadow: 24,
        p: 4,
    };

    onSwitch = (event: React.ChangeEvent<HTMLInputElement>) => {
        const { name, checked } = event.target;

        this.setState((prevState) => ({
            ...prevState,
            switchValues: prevState.switchValues.map((switchObj) => {
                if (switchObj.name === name) {
                    return {
                        ...switchObj,
                        value: checked
                    };
                }
                return switchObj;
            })
        }));
    }

    componentDidMount() {

    }

    componentWillUnmount() {

    }

    handleClose = () => {
        window.app.toggleWiFiSettings(false);
    }

    checkDependsValue = (depends: string) => {
        for(let x of this.state.switchValues)
        {
            if(x.name == depends)
                return x.value;
        }
    }

    render() {
        return <div className="setup">
            <Grid container spacing={2}>
                <Grid item xs={6}>
                    <FormControl component="fieldset" variant="standard">
                        <FormLabel component="legend">Regeln</FormLabel>
                        <FormGroup>
                            {Object.values(this.state.switchValues).map(value => {
                                return (
                                    <FormControlLabel sx={{mb: 1}}
                                                      control={
                                                          <Switch checked={(value.value && !value.depends) || (value.value && !!value.depends && this.checkDependsValue(value.depends))} disabled={!!value.depends && !this.checkDependsValue(value.depends)} onChange={this.onSwitch}
                                                                  name={value.name}/>
                                                      } label={value.label}
                                    />)
                            })}
                        </FormGroup>
                    </FormControl>
                </Grid>
                <Grid item xs={6}>
                    <FormControl component="fieldset" variant="standard">
                        <FormLabel component="legend">Standardwerte</FormLabel>
                        <FormGroup>

                        </FormGroup>
                    </FormControl>
                </Grid>
            </Grid>

        </div>
    }

}